from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from django.core.exceptions import ImproperlyConfigured

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_10_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.10",
    fixer_applied_from_version="1.10",
)


def _get_url_utils():
    """
    Get URL utilities through versions, despite them being moved and refactored (with new "path()" syntax).
    """
    try:
        from django.urls import (
            get_callable,
            RegexURLPattern,
            URLPattern,
            RegexURLResolver,
            URLResolver,
            NoReverseMatch,
        )
    except ImportError:
        # previously, there was no "RoutePattern vs RegexPattern"
        from django.core.urlresolvers import (
            get_callable,
            RegexURLPattern,
            RegexURLPattern as URLPattern,
            RegexURLResolver,
            RegexURLResolver as URLResolver,
            NoReverseMatch,
        )  # old location
    return (
        get_callable,
        RegexURLPattern,
        URLPattern,
        RegexURLResolver,
        URLResolver,
        NoReverseMatch,
    )


@register_django_compatibility_fixer(
    fixer_reference_version="1.10", fixer_applied_upto_version="1.10"
)
def fix_incoming_urls_submodule(utils):
    """
    Put a forward compatibility import path for django.urls, which replaces django.core.urlresolvers
    """
    from django.core import urlresolvers

    utils.inject_module("django.urls", urlresolvers)


@django1_10_bc_fixer()
def fix_deletion_templatetags_future(utils):
    """
    Preserve the "future" templatetags library, with its improved `firstof` and `cycle` tags.
    """
    import django.templatetags
    from ..django_legacy.django1_10.templatetags import future

    utils.inject_module("django.templatetags.future", future)
    utils.inject_attribute(django.templatetags, "future", future)

    from django.template.backends import django as django_templates

    _old_get_installed_libraries = django_templates.get_installed_libraries

    def get_installed_libraries():
        libraries = (
            _old_get_installed_libraries()
        )  # tries real __import__() calls on submodules
        libraries["future"] = "django.templatetags.future"
        # print(">>>>> FINAL libraries", libraries)
        return libraries

    utils.inject_callable(
        django_templates, "get_installed_libraries", get_installed_libraries
    )


@django1_10_bc_fixer()
def fix_deletion_template_defaulttags_ssi(utils):
    """
    Preserve the "ssi" default template tag.
    """
    from django.utils import six
    import django.template.defaulttags
    from ..django_legacy.django1_10.template import defaulttags

    utils.inject_callable(
        django.template.defaulttags,
        "include_is_allowed",
        defaulttags.include_is_allowed,
    )
    utils.inject_class(django.template.defaulttags, "SsiNode", defaulttags.SsiNode)
    utils.inject_callable(django.template.defaulttags, "ssi", defaulttags.ssi)
    django.template.defaulttags.register.tag(defaulttags.ssi)

    from django.template.engine import Engine

    _old_init = Engine.__init__

    def __init__(self, dirs=None, app_dirs=False, allowed_include_roots=None, **kwargs):
        if allowed_include_roots is None:
            allowed_include_roots = []
        if isinstance(allowed_include_roots, six.string_types):
            raise ImproperlyConfigured(
                "allowed_include_roots must be a tuple, not a string."
            )
        self.allowed_include_roots = allowed_include_roots
        _old_init(self, dirs=dirs, app_dirs=app_dirs, **kwargs)

    utils.inject_callable(Engine, "__init__", __init__)


@django1_10_bc_fixer()
def fix_behaviour_urls_resolvers_RegexURLPattern(utils):
    """
    Restore support for dotted-string view parameter in RegexURLPattern, instead passing a view object.
    """
    from django.utils import six

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del RegexURLPattern  # we work on the common URLPattern class

    @property
    def callback(self):
        callback = self.__dict__["callback"]  # bypass descriptor
        if isinstance(callback, six.string_types):
            callback_obj = get_callable(callback)
        else:
            callback_obj = callback
        return callback_obj

    @callback.setter
    def callback(self, value):
        self.__dict__["callback"] = value  # bypass descriptor

    # we inject a DATA-DESCRIPTOR, so it'll be accessed in prority
    # over "self.callback" instance attribute
    utils.inject_attribute(URLPattern, "callback", callback)

    def add_prefix(self, prefix):
        """
        Adds the prefix string to a string-based callback.
        """
        callback = self.__dict__["callback"]
        if not prefix or not isinstance(callback, six.string_types):
            return
        self.callback = prefix + "." + callback

    utils.inject_callable(URLPattern, "add_prefix", add_prefix)

    original_lookup_str = URLPattern.lookup_str

    @property  # not cached...
    def lookup_str(self):
        callback = self.__dict__["callback"]
        if isinstance(callback, six.string_types):
            # no need for warning, already emitted above
            return callback  # already a dotted path to view
        return original_lookup_str.__get__(self, self.__class__)

    utils.inject_attribute(URLPattern, "lookup_str", lookup_str)


@django1_10_bc_fixer()
def fix_behaviour_core_urlresolvers_reverse_with_prefix(utils):
    """
    Preserve the ability to call urlresolver on dotted string view,
    instead of explicit view name.
    """

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del RegexURLResolver  # we patch the common URLResolver class

    original_reverse_with_prefix = URLResolver._reverse_with_prefix

    def _reverse_with_prefix(self, lookup_view, _prefix, *args, **kwargs):
        original_lookup = lookup_view
        try:
            if self._is_callback(lookup_view):
                utils.emit_warning(
                    "Reversing by dotted path is deprecated (%s)." % original_lookup,
                    RemovedInDjango110Warning,
                    stacklevel=3,
                )
                lookup_view = get_callable(lookup_view)
        except (ImportError, AttributeError) as e:
            raise NoReverseMatch("Error importing '%s': %s." % (lookup_view, e))
        return original_reverse_with_prefix(self, lookup_view, _prefix, *args, **kwargs)

    utils.inject_callable(URLResolver, "_reverse_with_prefix", _reverse_with_prefix)


@django1_10_bc_fixer()
def fix_behaviour_conf_urls_url(utils):
    """
    Support passing views to url() as dotted strings instead of view objects.
    """
    from django.utils import six

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del URLPattern  # we stick to the old "regex-only" URL system

    from django.conf import urls

    def url(regex, view, kwargs=None, name=None, prefix=""):
        if isinstance(view, (list, tuple)):
            # For include(...) processing.
            urlconf_module, app_name, namespace = view
            return RegexURLResolver(
                regex, urlconf_module, kwargs, app_name=app_name, namespace=namespace
            )
        else:
            if isinstance(view, six.string_types):
                utils.emit_warning(
                    "Support for string view arguments to url() is deprecated and "
                    "will be removed in Django 1.10 (got %s). Pass the callable "
                    "instead." % view,
                    RemovedInDjango110Warning,
                    stacklevel=2,
                )
                if not view:
                    raise ImproperlyConfigured(
                        "Empty URL pattern view name not permitted (for pattern %r)"
                        % regex
                    )
                if prefix:
                    view = prefix + "." + view
            return RegexURLPattern(regex, view, kwargs, name)

    assert callable(urls.url)
    utils.inject_callable(urls, "url", url)


@django1_10_bc_fixer()
def fix_deletion_conf_urls_patterns(utils):
    """
    Preserve the patterns() builder for django urls.
    """
    from django.conf import urls

    get_callable, RegexURLPattern, URLPattern, RegexURLResolver, URLResolver, NoReverseMatch = (
        _get_url_utils()
    )
    del URLPattern  # we stick to the old "regex-only" URL system

    def patterns(prefix, *args):
        utils.emit_warning(
            "django.conf.urls.patterns() is deprecated and will be removed in "
            "Django 1.10. Update your urlpatterns to be a list of "
            "django.conf.urls.url() instances instead.",
            RemovedInDjango110Warning,
            stacklevel=2,
        )
        pattern_list = []
        for t in args:
            if isinstance(t, (list, tuple)):
                t = urls.url(prefix=prefix, *t)
            elif isinstance(t, RegexURLPattern):
                t.add_prefix(prefix)
            pattern_list.append(t)
        return pattern_list

    utils.inject_callable(urls, "patterns", patterns)

    urls.__all__.append(str("patterns"))  # so that star imports work fine


@django1_10_bc_fixer()
def fix_behaviour_template_smartif_OPERATORS_equals(utils):
    """
    Preserve support for a single '=' sign in {% if %} tag.
    """
    from django.template import smartif

    smartif.OPERATORS["="] = smartif.OPERATORS["=="]  # operator alias


''' REQUIRES PYTHON >= 3.3
@django1_10_bc_fixer()
def fix_deletion_core_context_processors(utils):
    """
    Keep django.core.context_processors middlewares as aliases for
    those of django.template.context_processors.
    """
    from .. import import_proxifier
    import_proxifier.install_module_alias_finder()  # idempotent
    import_proxifier.register_module_alias(module_alias="django.core.context_processors middlewares",
                                           real_module="django.template.context_processors")
'''


@django1_10_bc_fixer()
def fix_behaviour_core_management_parser_optparse(utils):
    """[UNSAFE] Preserve the support for old optparse instead of argparse parser, in management commands.

    Beware, Bash shell autocompletion might fail if some management commands use Optparse!
    """
    import sys
    from django.core.management import base, BaseCommand

    utils.inject_attribute(BaseCommand, "option_list", ())
    utils.inject_attribute(BaseCommand, "args", '')

    # ---

    @property
    def use_argparse(self):
        return not bool(self.option_list)
    utils.inject_attribute(BaseCommand, "use_argparse", use_argparse)

    def usage(self, subcommand):  # Predecessor of self.print_help()
        """
        Return a brief description of how to use this command, by
        default from the attribute ``self.help``.
        """
        usage = '%%prog %s [options] %s' % (subcommand, self.args)
        if self.help:
            return '%s\n\n%s' % (usage, self.help)
        else:
            return usage
    utils.inject_callable(BaseCommand, "usage", usage)

    # ---

    original_create_parser = BaseCommand.create_parser

    def create_parser(self, prog_name, subcommand, **kwargs):
        if not self.use_argparse:
            from optparse import OptionParser

            def store_as_int(option, opt_str, value, parser):
                setattr(parser.values, option.dest, int(value))

            # Backwards compatibility: use deprecated optparse module
            warnings.warn("OptionParser usage for Django management commands "
                          "is deprecated, use ArgumentParser instead",
                          RemovedInDjango110Warning)
            parser = OptionParser(prog=prog_name,
                                usage=self.usage(subcommand),
                                version=self.get_version())
            parser.add_option('-v', '--verbosity', action='callback', dest='verbosity', default=1,
                type='choice', choices=['0', '1', '2', '3'], callback=store_as_int,
                help='Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output')
            parser.add_option('--settings',
                help=(
                    'The Python path to a settings module, e.g. '
                    '"myproject.settings.main". If this isn\'t provided, the '
                    'DJANGO_SETTINGS_MODULE environment variable will be used.'
                ),
            )
            parser.add_option('--pythonpath',
                help='A directory to add to the Python path, e.g. "/home/djangoprojects/myproject".'),
            parser.add_option('--traceback', action='store_true',
                help='Raise on CommandError exceptions')
            parser.add_option('--no-color', action='store_true', dest='no_color', default=False,
                help="Don't colorize the command output.")
            parser.add_option('--force-color', action='store_true',  # ADDED to old code
                help='FORWARD-COMPATIBILITY OPTION FOR OLD OPTPARSE PARSER')
            parser.add_option('--skip-checks', action='store_true',  # ADDED to old code
                help='FORWARD-COMPATIBILITY OPTION FOR OLD OPTPARSE PARSER')
            for opt in self.option_list:
                parser.add_option(opt)
        else:
            parser = original_create_parser(self, prog_name, subcommand)
            if self.args:
                # Keep compatibility and always accept positional arguments, like optparse when args is set
                parser.add_argument('args', nargs='*')
        return parser
    utils.inject_callable(BaseCommand, "create_parser", create_parser)

    # ---

    original_run_from_argv = BaseCommand.run_from_argv

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path
        and Django settings), then run this command. If the
        command raises a ``CommandError``, intercept it and print it sensibly
        to stderr. If the ``--traceback`` option is present or the raised
        ``Exception`` is not ``CommandError``, raise it.
        """

        if self.use_argparse:
            original_run_from_argv(self, argv)
            return

        # Fallback for OPTPARSE #
        self._called_from_command_line = True
        parser = self.create_parser(argv[0], argv[1])
        options, args = parser.parse_args(argv[2:])
        cmd_options = vars(options)

        # Copy of the remainder of the original run_from_argv() method
        base.handle_default_options(options)
        try:
            self.execute(*args, **cmd_options)
        except Exception as e:
            if options.traceback or not isinstance(e, base.CommandError):
                raise

            # SystemCheckError takes care of its own formatting.
            if isinstance(e, base.SystemCheckError):
                self.stderr.write(str(e), lambda x: x)
            else:
                self.stderr.write('%s: %s' % (e.__class__.__name__, e))
            sys.exit(1)
        finally:
            base.connections.close_all()
    utils.inject_callable(BaseCommand, "run_from_argv", run_from_argv)

    # ---

    from django.core import management
    original_call_command = management.call_command

    def call_command(name, *args, **options):
        try:
            return original_call_command(name, *args, **options)
        except AttributeError as exc:
            # We expect something like "AttributeError: 'OptionParser' object has no attribute '_actions'"

            if "OptionParser" not in str(exc):
                raise  # It's an unrelated exception

            # FALLBACK TO OPTPARSE PARSER, by reproducing call_command() code

            # Load the command object.
            try:
                app_name = management.get_commands()[name]
            except KeyError:
                raise management.CommandError("Unknown command: %r" % name)

            if isinstance(app_name, BaseCommand):
                # If the command is already loaded, use it directly.
                command = app_name
            else:
                command = management.load_command_class(app_name, name)

            # Simulate argument parsing to get the option defaults (see #10080 for details).
            parser = command.create_parser('', name)
            # Legacy optparse method
            defaults, _ = parser.parse_args(args=[])
            defaults = dict(defaults.__dict__, **options)
            if 'skip_checks' not in options:
                defaults['skip_checks'] = True

            return command.execute(*args, **defaults)
    utils.inject_callable(management, "call_command", call_command)



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_7_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.7",
    fixer_applied_from_version="1.7",
)



from __future__ import absolute_import, print_function, unicode_literals

import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_project.settings")

import django
import django_compat_patcher

# This MUST happen BEFORE django is setup(), else import proxies can't be installed
settings = dict(DCP_EXCLUDE_FIXER_IDS=[])  # REMOVE the disabling of unsafe fixers, for tests!
django_compat_patcher.patch(settings)

django.setup()  # idempotent

DJANGO_VERSION_TUPLE = django.VERSION







from __future__ import absolute_import, print_function, unicode_literals

import json
from os import getenv

class DjangoSettingsProvider(object):

    _settings = None

    def __init__(self, settings):
        """
        Either directly provided `settings` parameter (if set) or
        Django project settings are used, but no fallback occurs between them.

        Environment variables (in JSON format) have precedence over all DCP settings.
        """
        if settings:
            self._settings = settings
        else:
            from django.conf import settings as django_settings
            self._settings = django_settings

        from . import default_settings as default_settings

        self._fallback_settings = default_settings.__dict__

    def __getitem__(self, item):
        """Might raise KeyError if setting is not existing at all"""

        item = "DCP_" + item.upper()  # Turn to Django naming convention

        value = getenv(item, default=None)
        if value is not None:
            try:
                return json.loads(value)
            except ValueError as exc:
                raise ValueError("Abnormal json value %r for env variable %s (%s)" % (value, item, exc))

        try:
            value = self._settings[item] if isinstance(self._settings, dict) else getattr(self._settings, item)
        except (KeyError, AttributeError) as e:
            # print("EXCEPTION IN get_patcher_setting", name, e)
            value = self._fallback_settings[item]
        return value



import copy
import datetime
import os
import pickle
import time
import unittest
import six

from django.test import SimpleTestCase


class SortedDictTests(SimpleTestCase):
    def setUp(self):
        from django.utils.datastructures import SortedDict

        super(SortedDictTests, self).setUp()
        self.d1 = SortedDict()
        self.d1[7] = "seven"
        self.d1[1] = "one"
        self.d1[9] = "nine"

        self.d2 = SortedDict()
        self.d2[1] = "one"
        self.d2[9] = "nine"
        self.d2[0] = "nil"
        self.d2[7] = "seven"

    def test_basic_methods(self):
        self.assertEqual(list(six.iterkeys(self.d1)), [7, 1, 9])
        self.assertEqual(list(six.itervalues(self.d1)), ["seven", "one", "nine"])
        self.assertEqual(
            list(six.iteritems(self.d1)), [(7, "seven"), (1, "one"), (9, "nine")]
        )

    def test_overwrite_ordering(self):
        """ Overwriting an item keeps its place. """
        self.d1[1] = "ONE"
        self.assertEqual(list(six.itervalues(self.d1)), ["seven", "ONE", "nine"])

    def test_append_items(self):
        """ New items go to the end. """
        self.d1[0] = "nil"
        self.assertEqual(list(six.iterkeys(self.d1)), [7, 1, 9, 0])

    def test_delete_and_insert(self):
        """
        Deleting an item, then inserting the same key again will place it
        at the end.
        """
        del self.d2[7]
        self.assertEqual(list(six.iterkeys(self.d2)), [1, 9, 0])
        self.d2[7] = "lucky number 7"
        self.assertEqual(list(six.iterkeys(self.d2)), [1, 9, 0, 7])

    if six.PY2:

        def test_change_keys(self):
            """
            Changing the keys won't do anything, it's only a copy of the
            keys dict.
            This test doesn't make sense under Python 3 because keys is
            an iterator.
            """
            k = self.d2.keys()
            k.remove(9)
            self.assertEqual(self.d2.keys(), [1, 9, 0, 7])

    def test_init_keys(self):
        """
        Initialising a SortedDict with two keys will just take the first one.
        A real dict will actually take the second value so we will too, but
        we'll keep the ordering from the first key found.
        """
        from django.utils.datastructures import SortedDict

        tuples = ((2, "two"), (1, "one"), (2, "second-two"))
        d = SortedDict(tuples)

        self.assertEqual(list(six.iterkeys(d)), [2, 1])

        real_dict = dict(tuples)
        self.assertEqual(sorted(six.itervalues(real_dict)), ["one", "second-two"])

        # Here the order of SortedDict values *is* what we are testing
        self.assertEqual(list(six.itervalues(d)), ["second-two", "one"])

    def test_overwrite(self):
        self.d1[1] = "not one"
        self.assertEqual(self.d1[1], "not one")
        self.assertEqual(
            list(six.iterkeys(self.d1)), list(six.iterkeys(self.d1.copy()))
        )

    def test_append(self):
        self.d1[13] = "thirteen"
        self.assertEqual(
            repr(self.d1), "{7: 'seven', 1: 'one', 9: 'nine', 13: 'thirteen'}"
        )

    def test_pop(self):
        self.assertEqual(self.d1.pop(1, "missing"), "one")
        self.assertEqual(self.d1.pop(1, "missing"), "missing")

        # We don't know which item will be popped in popitem(), so we'll
        # just check that the number of keys has decreased.
        l = len(self.d1)
        self.d1.popitem()
        self.assertEqual(l - len(self.d1), 1)

    def test_dict_equality(self):
        from django.utils.datastructures import SortedDict

        d = SortedDict((i, i) for i in range(3))
        self.assertEqual(d, {0: 0, 1: 1, 2: 2})

    def test_tuple_init(self):
        from django.utils.datastructures import SortedDict

        d = SortedDict(((1, "one"), (0, "zero"), (2, "two")))
        self.assertEqual(repr(d), "{1: 'one', 0: 'zero', 2: 'two'}")

    def test_pickle(self):
        self.assertEqual(
            pickle.loads(pickle.dumps(self.d1, 2)), {7: "seven", 1: "one", 9: "nine"}
        )

    def test_copy(self):
        from django.utils.datastructures import SortedDict

        orig = SortedDict(((1, "one"), (0, "zero"), (2, "two")))
        copied = copy.copy(orig)
        self.assertEqual(list(six.iterkeys(orig)), [1, 0, 2])
        self.assertEqual(list(six.iterkeys(copied)), [1, 0, 2])

    def test_clear(self):
        self.d1.clear()
        self.assertEqual(self.d1, {})
        self.assertEqual(self.d1.keyOrder, [])

    def test_reversed(self):
        self.assertEqual(list(self.d1), [7, 1, 9])
        self.assertEqual(list(self.d2), [1, 9, 0, 7])
        self.assertEqual(list(reversed(self.d1)), [9, 1, 7])
        self.assertEqual(list(reversed(self.d2)), [7, 0, 9, 1])


class MergeDictTests(SimpleTestCase):
    def test_simple_mergedict(self):
        from django.utils.datastructures import MergeDict

        d1 = {
            "chris": "cool",
            "camri": "cute",
            "cotton": "adorable",
            "tulip": "snuggable",
            "twoofme": "firstone",
        }

        d2 = {
            "chris2": "cool2",
            "camri2": "cute2",
            "cotton2": "adorable2",
            "tulip2": "snuggable2",
        }

        d3 = {
            "chris3": "cool3",
            "camri3": "cute3",
            "cotton3": "adorable3",
            "tulip3": "snuggable3",
        }

        md = MergeDict(d1, d2, d3)

        self.assertEqual(md["chris"], "cool")
        self.assertEqual(md["camri"], "cute")
        self.assertEqual(md["twoofme"], "firstone")

        md2 = md.copy()
        self.assertEqual(md2["chris"], "cool")

    def test_mergedict_merges_multivaluedict(self):
        """ MergeDict can merge MultiValueDicts """
        from django.utils.datastructures import MergeDict, MultiValueDict

        multi1 = MultiValueDict({"key1": ["value1"], "key2": ["value2", "value3"]})

        multi2 = MultiValueDict({"key2": ["value4"], "key4": ["value5", "value6"]})

        mm = MergeDict(multi1, multi2)

        # Although 'key2' appears in both dictionaries,
        # only the first value is used.
        self.assertEqual(mm.getlist("key2"), ["value2", "value3"])
        self.assertEqual(mm.getlist("key4"), ["value5", "value6"])
        self.assertEqual(mm.getlist("undefined"), [])

        self.assertEqual(sorted(six.iterkeys(mm)), ["key1", "key2", "key4"])
        self.assertEqual(len(list(six.itervalues(mm))), 3)

        self.assertIn("value1", six.itervalues(mm))

        self.assertEqual(
            sorted(six.iteritems(mm), key=lambda k: k[0]),
            [("key1", "value1"), ("key2", "value3"), ("key4", "value6")],
        )

        self.assertEqual(
            [(k, mm.getlist(k)) for k in sorted(mm)],
            [
                ("key1", ["value1"]),
                ("key2", ["value2", "value3"]),
                ("key4", ["value5", "value6"]),
            ],
        )

    def test_bool_casting(self):
        from django.utils.datastructures import MergeDict

        empty = MergeDict({}, {}, {})
        not_empty = MergeDict({}, {}, {"key": "value"})
        self.assertFalse(empty)
        self.assertTrue(not_empty)

    def test_key_error(self):
        """
        Test that the message of KeyError contains the missing key name.
        """
        from django.utils.datastructures import MergeDict

        d1 = MergeDict({"key1": 42})
        with six.assertRaisesRegex(self, KeyError, "key2"):
            d1["key2"]


class TzinfoTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(TzinfoTests, cls).setUpClass()
        cls.old_TZ = os.environ.get("TZ")
        os.environ["TZ"] = "US/Eastern"

        try:
            # Check if a timezone has been set
            time.tzset()
            cls.tz_tests = True
        except AttributeError:
            # No timezone available. Don't run the tests that require a TZ
            cls.tz_tests = False

    @classmethod
    def tearDownClass(cls):
        if cls.old_TZ is None:
            del os.environ["TZ"]
        else:
            os.environ["TZ"] = cls.old_TZ

        # Cleanup - force re-evaluation of TZ environment variable.
        if cls.tz_tests:
            time.tzset()
        super(TzinfoTests, cls).tearDownClass()

    def test_fixedoffset(self):
        from django.utils.tzinfo import FixedOffset

        self.assertEqual(repr(FixedOffset(0)), "+0000")
        self.assertEqual(repr(FixedOffset(60)), "+0100")
        self.assertEqual(repr(FixedOffset(-60)), "-0100")
        self.assertEqual(repr(FixedOffset(280)), "+0440")
        self.assertEqual(repr(FixedOffset(-280)), "-0440")
        self.assertEqual(repr(FixedOffset(-78.4)), "-0118")
        self.assertEqual(repr(FixedOffset(78.4)), "+0118")
        self.assertEqual(repr(FixedOffset(-5.5 * 60)), "-0530")
        self.assertEqual(repr(FixedOffset(5.5 * 60)), "+0530")
        self.assertEqual(repr(FixedOffset(-0.5 * 60)), "-0030")
        self.assertEqual(repr(FixedOffset(0.5 * 60)), "+0030")

    def test_16899(self):
        if not self.tz_tests:
            return
        from django.utils.tzinfo import LocalTimezone

        ts = 1289106000
        # Midnight at the end of DST in US/Eastern: 2010-11-07T05:00:00Z
        dt = datetime.datetime.utcfromtimestamp(ts)
        # US/Eastern -- we force its representation to "EST"
        tz = LocalTimezone(dt + datetime.timedelta(days=1))
        self.assertEqual(
            repr(datetime.datetime.fromtimestamp(ts - 3600, tz)),
            "datetime.datetime(2010, 11, 7, 0, 0, tzinfo=EST)",
        )
        self.assertEqual(
            repr(datetime.datetime.fromtimestamp(ts, tz)),
            "datetime.datetime(2010, 11, 7, 1, 0, tzinfo=EST)",
        )
        self.assertEqual(
            repr(datetime.datetime.fromtimestamp(ts + 3600, tz)),
            "datetime.datetime(2010, 11, 7, 1, 0, tzinfo=EST)",
        )

    def test_copy(self):
        from django.utils.tzinfo import FixedOffset, LocalTimezone

        now = datetime.datetime.now()
        self.assertIsInstance(copy.copy(FixedOffset(90)), FixedOffset)
        self.assertIsInstance(copy.copy(LocalTimezone(now)), LocalTimezone)

    def test_deepcopy(self):
        from django.utils.tzinfo import FixedOffset, LocalTimezone

        now = datetime.datetime.now()
        self.assertIsInstance(copy.deepcopy(FixedOffset(90)), FixedOffset)
        self.assertIsInstance(copy.deepcopy(LocalTimezone(now)), LocalTimezone)

    def test_pickling_unpickling(self):
        from django.utils.tzinfo import FixedOffset, LocalTimezone

        now = datetime.datetime.now()
        self.assertIsInstance(pickle.loads(pickle.dumps(FixedOffset(90))), FixedOffset)
        self.assertIsInstance(
            pickle.loads(pickle.dumps(LocalTimezone(now))), LocalTimezone
        )



from django.db import models


class SimpleModel(models.Model):
    name = models.CharField(max_length=100)
    age = models.IntegerField(default=0)
    is_active = models.BooleanField(default=True)


class SimpleModelChild(models.Model):
    name = models.CharField(max_length=100)
    parent = models.ForeignKey(SimpleModel, null=True, on_delete=models.CASCADE)



import importlib
import itertools
import json
import os

from django import http
from django.apps import apps
from django.template import Context, Engine
from django.utils import six
from django.utils._os import upath
from django.utils.translation import (
    check_for_language, get_language, to_locale,
)
from django.utils.translation.trans_real import DjangoTranslation
from django.views.i18n import js_catalog_template, get_formats

from django_compat_patcher.deprecation import *

DEFAULT_PACKAGES = ['django.conf']
LANGUAGE_QUERY_PARAMETER = 'language'


def render_javascript_catalog(catalog=None, plural=None):
    template = Engine().from_string(js_catalog_template)

    def indent(s):
        return s.replace('\n', '\n  ')

    context = Context({
        'catalog_str': indent(json.dumps(
            catalog, sort_keys=True, indent=2)) if catalog else None,
        'formats_str': indent(json.dumps(
            get_formats(), sort_keys=True, indent=2)),
        'plural': plural,
    })

    return http.HttpResponse(template.render(context), 'text/javascript')



def get_javascript_catalog(locale, domain, packages):
    app_configs = apps.get_app_configs()
    allowable_packages = set(app_config.name for app_config in app_configs)
    allowable_packages.update(DEFAULT_PACKAGES)
    packages = [p for p in packages if p in allowable_packages]
    paths = []
    # paths of requested packages
    for package in packages:
        p = importlib.import_module(package)
        path = os.path.join(os.path.dirname(upath(p.__file__)), 'locale')
        paths.append(path)

    trans = DjangoTranslation(locale, domain=domain, localedirs=paths)
    trans_cat = trans._catalog

    plural = None
    if '' in trans_cat:
        for line in trans_cat[''].split('\n'):
            if line.startswith('Plural-Forms:'):
                plural = line.split(':', 1)[1].strip()
    if plural is not None:
        # this should actually be a compiled function of a typical plural-form:
        # Plural-Forms: nplurals=3; plural=n%10==1 && n%100!=11 ? 0 :
        #               n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;
        plural = [el.strip() for el in plural.split(';') if el.strip().startswith('plural=')][0].split('=', 1)[1]

    pdict = {}
    maxcnts = {}
    catalog = {}
    trans_fallback_cat = trans._fallback._catalog if trans._fallback else {}
    for key, value in itertools.chain(six.iteritems(trans_cat), six.iteritems(trans_fallback_cat)):
        if key == '' or key in catalog:
            continue
        if isinstance(key, six.string_types):
            catalog[key] = value
        elif isinstance(key, tuple):
            msgid = key[0]
            cnt = key[1]
            maxcnts[msgid] = max(cnt, maxcnts.get(msgid, 0))
            pdict.setdefault(msgid, {})[cnt] = value
        else:
            raise TypeError(key)
    for k, v in pdict.items():
        catalog[k] = [v.get(i, '') for i in range(maxcnts[k] + 1)]

    return catalog, plural


def _get_locale(request):
    language = request.GET.get(LANGUAGE_QUERY_PARAMETER)
    if not (language and check_for_language(language)):
        language = get_language()
    return to_locale(language)


def _parse_packages(packages):
    if packages is None:
        packages = list(DEFAULT_PACKAGES)
    elif isinstance(packages, six.string_types):
        packages = packages.split('+')
    return packages


def null_javascript_catalog(request, domain=None, packages=None):
    """
    Returns "identity" versions of the JavaScript i18n functions -- i.e.,
    versions that don't actually do anything.
    """
    return render_javascript_catalog()


def javascript_catalog(request, domain='djangojs', packages=None):
    """
    Returns the selected language catalog as a javascript library.

    Receives the list of packages to check for translations in the
    packages parameter either from an infodict or as a +-delimited
    string from the request. Default is 'django.conf'.

    Additionally you can override the gettext domain for this view,
    but usually you don't want to do that, as JavaScript messages
    go to the djangojs domain. But this might be needed if you
    deliver your JavaScript source from Django templates.
    """
    warnings.warn(
        "The javascript_catalog() view is deprecated in favor of the "
        "JavaScriptCatalog view.", RemovedInDjango20Warning, stacklevel=2
    )
    locale = _get_locale(request)
    packages = _parse_packages(packages)
    catalog, plural = get_javascript_catalog(locale, domain, packages)
    return render_javascript_catalog(catalog, plural)


def json_catalog(request, domain='djangojs', packages=None):
    """
    Return the selected language catalog as a JSON object.

    Receives the same parameters as javascript_catalog(), but returns
    a response with a JSON object of the following format:

        {
            "catalog": {
                # Translations catalog
            },
            "formats": {
                # Language formats for date, time, etc.
            },
            "plural": '...'  # Expression for plural forms, or null.
        }
    """
    warnings.warn(
        "The json_catalog() view is deprecated in favor of the "
        "JSONCatalog view.", RemovedInDjango20Warning, stacklevel=2
    )
    locale = _get_locale(request)
    packages = _parse_packages(packages)
    catalog, plural = get_javascript_catalog(locale, domain, packages)
    data = {
        'catalog': catalog,
        'formats': get_formats(),
        'plural': plural,
    }
    return http.JsonResponse(data)



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_6_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.6",
    fixer_applied_from_version="1.6",
)


@django1_6_bc_fixer()
def fix_deletion_http_request_HttpRequest_raw_post_data(utils):
    """
    Preserve the request.raw_post_data alias for request.body.
    """
    from django.http.request import HttpRequest

    @property
    def raw_post_data(self):
        utils.emit_warning(
            "HttpRequest.raw_post_data has been deprecated. Use HttpRequest.body instead.",
            RemovedInDjango16Warning,
        )
        return self.body

    utils.inject_attribute(HttpRequest, "raw_post_data", raw_post_data)



from __future__ import absolute_import, print_function, unicode_literals

import os
import sys



sys.path.append(os.path.dirname(os.path.abspath(__file__)))

os.environ["DJANGO_SETTINGS_MODULE"] = "test_project.minimal_settings"

os.environ.update(
    DCP_INCLUDE_FIXER_IDS = '["some_included_id"]',
    DCP_INCLUDE_FIXER_FAMILIES = '["some_included_family"]',

    DCP_EXCLUDE_FIXER_IDS = '["some_excluded_id"]',
    DCP_EXCLUDE_FIXER_FAMILIES = '["some_excluded_family"]',

    DCP_PATCH_INJECTED_OBJECTS = 'true',

    DCP_LOGGING_LEVEL = '"DEBUG"',
    DCP_ENABLE_WARNINGS = 'false',
)

import django
import django_compat_patcher

django_compat_patcher.patch()  # fixers which can't apply should be skipped, not crash

from django_compat_patcher.config import DjangoSettingsProvider

django_settings_provider = DjangoSettingsProvider(settings=None)

assert django_settings_provider["include_fixer_ids"] == ["some_included_id"]
assert django_settings_provider["include_fixer_families"] == ["some_included_family"]
assert django_settings_provider["exclude_fixer_ids"] == ["some_excluded_id"]
assert django_settings_provider["exclude_fixer_families"] == ["some_excluded_family"]
assert django_settings_provider["patch_injected_objects"] == True
assert django_settings_provider["logging_level"] == "DEBUG"
assert django_settings_provider["enable_warnings"] == False



from __future__ import absolute_import, print_function, unicode_literals

from . import (
    django1_6,
    django1_7,
    django1_8,
    django1_9,
    django1_10,
    django1_11,
    django2_0,
    django2_1,
    django2_2,
    django3_0,
    django3_1,
)



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_20_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="2.0",
    fixer_applied_from_version="2.0",
)


# This change should not be patched, since security issues could ensue:
# "Using User.is_authenticated() and User.is_anonymous() as methods rather than properties is no longer supported."

# Preserving mark_for_escaping() and related classes would be complicated and might raise security issues, so it's not done for now.


@django1_20_bc_fixer()
def fix_deletion_urls_RegexURLPattern_RegexURLResolver(utils):
    """
    Preserve RegexURLPattern and RegexURLResolver in django.urls, which disappeared due to DEP 0201.
    """
    import django.urls.resolvers
    from django.urls.resolvers import RegexPattern, URLPattern, URLResolver

    class RegexURLPattern(URLPattern):
        def __init__(self, pattern, *args, **kwargs):
            # we force is_endpoint else Warnings about "$" ends in regexes...
            URLPattern.__init__(
                self, RegexPattern(pattern, is_endpoint=True), *args, **kwargs
            )

    utils.inject_class(django.urls.resolvers, "RegexURLPattern", RegexURLPattern)
    utils.inject_class(django.urls, "RegexURLPattern", RegexURLPattern)

    class RegexURLResolver(URLResolver):
        def __init__(self, pattern, *args, **kwargs):
            URLResolver.__init__(self, RegexPattern(pattern), *args, **kwargs)

    utils.inject_class(django.urls.resolvers, "RegexURLResolver", RegexURLResolver)
    utils.inject_class(django.urls, "RegexURLResolver", RegexURLResolver)


@django1_20_bc_fixer()
def fix_deletion_core_urlresolvers(utils):
    """
    Preserve django.core.urlresolvers module, now replaced by django.urls.
    """
    from django import urls

    utils.inject_module("django.core.urlresolvers", urls)


@django1_20_bc_fixer()
def fix_deletion_template_library_assignment_tag(utils):
    """
    Preserve the assignment_tag() helper, superseded by simple_tag().
    """
    import django.template.library

    def assignment_tag(self, func=None, takes_context=None, name=None):
        utils.emit_warning(
            "assignment_tag() is deprecated. Use simple_tag() instead",
            RemovedInDjango20Warning,
            stacklevel=2,
        )
        return self.simple_tag(func, takes_context, name)

    utils.inject_callable(
        django.template.library.Library, "assignment_tag", assignment_tag
    )


@django1_20_bc_fixer()
def fix_deletion_utils_functional_allow_lazy(utils):
    """
    Preserve the allow_lazy() utility, superseded by keep_lazy().
    """
    import django.utils.functional

    def allow_lazy(func, *resultclasses):
        from django.utils.functional import keep_lazy

        utils.emit_warning(
            "django.utils.functional.allow_lazy() is deprecated in favor of "
            "django.utils.functional.keep_lazy()",
            RemovedInDjango20Warning,
            2,
        )
        return keep_lazy(*resultclasses)(func)

    utils.inject_callable(django.utils.functional, "allow_lazy", allow_lazy)


@django1_20_bc_fixer()
def fix_deletion_template_context_Context_has_key(utils):
    """
    Preserve the Context.has_key() utility, replaced by "in" operator use.
    """
    import django.template.context

    def has_key(self, key):
        utils.emit_warning(
            "%s.has_key() is deprecated in favor of the 'in' operator."
            % self.__class__.__name__,
            RemovedInDjango20Warning,
        )
        return key in self

    utils.inject_callable(django.template.context.Context, "has_key", has_key)


@django1_20_bc_fixer()
def fix_deletion_views_i18n_javascript_and_json_catalog(utils):
    """
    Preserve the javascript_catalog() and json_catalog() i18n views, superseded by class-based views.
    """

    from django_compat_patcher.django_legacy.django2_0.views.i18n import (
        javascript_catalog,
        json_catalog,
        render_javascript_catalog,
        null_javascript_catalog,
    )

    import django.views.i18n

    utils.inject_callable(django.views.i18n, "javascript_catalog", javascript_catalog)
    utils.inject_callable(django.views.i18n, "json_catalog", json_catalog)
    utils.inject_callable(
        django.views.i18n, "render_javascript_catalog", render_javascript_catalog
    )
    utils.inject_callable(
        django.views.i18n, "null_javascript_catalog", null_javascript_catalog
    )


@django1_20_bc_fixer()
def fix_behaviour_db_models_fields_related_ForeignKey_OneToOneField(utils):
    """
    Let "on_delete" parameter of ForeignKey and OneToOneField be optional, defaulting to CASCADE.
    """
    from django.db.models import ForeignKey, OneToOneField, CASCADE

    original_ForeignKey_init = ForeignKey.__dict__["__init__"]

    def __init_ForeignKey__(
        self,
        to,
        on_delete=None,
        related_name=None,
        related_query_name=None,
        limit_choices_to=None,
        parent_link=False,
        to_field=None,
        db_constraint=True,
        **kwargs
    ):

        if on_delete is None:
            utils.emit_warning(
                "on_delete will be a required arg for %s in Django 2.0. Set "
                "it to models.CASCADE on models and in existing migrations "
                "if you want to maintain the current default behavior. ",
                RemovedInDjango20Warning,
                2,
            )
            on_delete = CASCADE

        original_ForeignKey_init(
            self,
            to,
            on_delete=on_delete,
            related_name=related_name,
            related_query_name=related_query_name,
            limit_choices_to=limit_choices_to,
            parent_link=parent_link,
            to_field=to_field,
            db_constraint=db_constraint,
            **kwargs
        )

    utils.inject_callable(ForeignKey, "__init__", __init_ForeignKey__)

    original_OneToOneField_init = OneToOneField.__dict__["__init__"]

    def __init_OneToOneField__(self, to, on_delete=None, to_field=None, **kwargs):

        if on_delete is None:
            utils.emit_warning(
                "on_delete will be a required arg for %s in Django 2.0. Set "
                "it to models.CASCADE on models and in existing migrations "
                "if you want to maintain the current default behavior. ",
                RemovedInDjango20Warning,
                2,
            )
            on_delete = CASCADE

        original_OneToOneField_init(
            self, to, on_delete=on_delete, to_field=to_field, **kwargs
        )

    utils.inject_callable(OneToOneField, "__init__", __init_OneToOneField__)


@django1_20_bc_fixer()
def fix_behaviour_conf_urls_include_3tuples(utils):
    """
    Keep accepting a 3-tuple (urlconf_module, app_name, namespace) as first argument of include(),
    instead of providing namespace argument directly to include()
    """

    from django.core.exceptions import ImproperlyConfigured
    from django.conf.urls import include as original_include

    def include(arg, namespace=None):
        if isinstance(arg, tuple):
            if len(arg) == 3:
                if namespace:
                    raise ImproperlyConfigured(
                        "Cannot override the namespace for a dynamic module that provides a namespace"
                    )
                utils.emit_warning(
                    "Passing a 3-tuple to django.conf.urls.include() is deprecated. "
                    "Pass a 2-tuple containing the list of patterns and app_name, "
                    "and provide the namespace argument to include() instead.",
                    RemovedInDjango20Warning,
                    stacklevel=2,
                )
                urlconf_module, app_name, namespace = arg
                arg = (urlconf_module, app_name)
        return original_include(arg, namespace=namespace)

    import django.conf.urls

    utils.inject_callable(django.conf.urls, "include", include)
    import django.urls

    utils.inject_callable(django.urls, "include", include)


@django1_20_bc_fixer(fixer_delayed=True)  # "User" model might be unavailable before setup() is done!
def fix_behaviour_contrib_auth_user_is_anonymous_is_authenticated_callability(utils):
    """
    Make user.is_anonymous and user.is_authenticated behave both as properties and methods,
    by preserving their callability like in earlier Django version.
    """
    utils.skip_if_app_not_installed("django.contrib.contenttypes")  # BEFORE IMPORTS!

    from django.contrib.auth.base_user import AbstractBaseUser
    from django.contrib.auth.models import AnonymousUser
    from ..django_legacy.django2_0.utils.deprecation import CallableFalse, CallableTrue

    @property
    def is_anonymous_for_AbstractBaseUser(self):
        """
        Always return False. This is a way of comparing User objects to
        anonymous users.
        """
        return CallableFalse

    @property
    def is_authenticated_for_AbstractBaseUser(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in templates.
        """
        return CallableTrue

    utils.inject_attribute(AbstractBaseUser, "is_anonymous", is_anonymous_for_AbstractBaseUser)
    utils.inject_attribute(AbstractBaseUser, "is_authenticated", is_authenticated_for_AbstractBaseUser)

    @property
    def is_anonymous_for_AnonymousUser(self):
        return CallableTrue

    @property
    def is_authenticated_for_AnonymousUser(self):
        return CallableFalse

    utils.inject_attribute(AnonymousUser, "is_anonymous", is_anonymous_for_AnonymousUser)
    utils.inject_attribute(AnonymousUser, "is_authenticated", is_authenticated_for_AnonymousUser)


@django1_20_bc_fixer()
def fix_behaviour_db_models_fields_related_descriptors_ReverseManyToOneDescriptor_setter(utils):
    """
    Restore support for direct assignment to the reverse side of a related set, in many-to-one and many-to-many relationships.
    """
    from django.db.models.fields.related_descriptors import ReverseManyToOneDescriptor

    def __set__(self, instance, value):
        """
        Set the related objects through the reverse relation.

        With the example above, when setting ``parent.children = children``:

        - ``self`` is the descriptor managing the ``children`` attribute
        - ``instance`` is the ``parent`` instance
        - ``value`` is the ``children`` sequence on the right of the equal sign
        """
        utils.emit_warning(
            'Direct assignment to the %s is deprecated due to the implicit '
            'save() that happens. Use %s.set() instead.' % self._get_set_deprecation_msg_params(),
            RemovedInDjango20Warning, stacklevel=2,
        )
        manager = self.__get__(instance)
        manager.set(value)
        print(">>>>>>>>> WE HAVE SET MANAGER TO", value)

    ReverseManyToOneDescriptor.__set__ = __set__



from __future__ import absolute_import, print_function, unicode_literals

from compat_patcher_core.runner import PatchingRunner


class DjangoPatchingRunner(PatchingRunner):
    def patch_software(self, settings=None):
        """For Django, we separate fixers between those who apply before and after django.setup()."""

        relevant_fixers = self._get_sorted_relevant_fixers()

        # print("> relevant_fixers", relevant_fixers)
        pre_fixers = [
            f for f in relevant_fixers if "fixer_delayed" not in f["fixer_tags"]
        ]
        post_fixers = [f for f in relevant_fixers if "fixer_delayed" in f["fixer_tags"]]
        assert len(relevant_fixers) == len(pre_fixers) + len(post_fixers)

        fixers_just_applied = self._apply_selected_fixers(pre_fixers)
        import django

        django.setup()  # Theoretically idempotent (except regarding logging?)
        fixers_just_applied += self._apply_selected_fixers(post_fixers)

        # Be consistent with origianl signature
        return dict(fixers_just_applied=fixers_just_applied)



from __future__ import absolute_import, print_function, unicode_literals


# NOTE : No fix provided for "Admin actions are no longer collected from base ModelAdmin classes" for now, since it seems no one really relied on this behaviour?




from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_31_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="3.1",
    fixer_applied_from_version="3.1",
)


@django1_31_bc_fixer()
def fix_deletion_db_models_submodules_EmptyResultSet(utils):
    """Preserve compatibility imports of django.core.exceptions.EmptyResultSet in
    django.db.models.query, django.db.models.sql, and django.db.models.sql.datastructures
    """
    from django.core.exceptions import EmptyResultSet
    from django.db.models import query
    from django.db.models import sql
    from django.db.models.sql import datastructures

    utils.inject_class(query, "EmptyResultSet", EmptyResultSet)
    utils.inject_class(sql, "EmptyResultSet", EmptyResultSet)
    utils.inject_class(datastructures, "EmptyResultSet", EmptyResultSet)

    sql.__all__.append("EmptyResultSet")  # Preserve star import


@django1_31_bc_fixer()
def fix_deletion_db_models_fields_FieldDoesNotExist(utils):
    """
    Preserve compatibility import of django.core.exceptions.FieldDoesNotExist in django.db.models.fields
    """
    from django.core.exceptions import FieldDoesNotExist
    from django.db.models import fields

    utils.inject_class(fields, "FieldDoesNotExist", FieldDoesNotExist)
    fields.__all__.append("FieldDoesNotExist")  # Preserve star import


@django1_31_bc_fixer()
def fix_deletion_forms_forms_pretty_name_BoundField(utils):
    """
    Preserve the compatibility imports of django.forms.utils.pretty_name() and
    django.forms.boundfield.BoundField in django.forms.forms
    """
    from django.forms import forms

    from django.forms.boundfield import BoundField
    from django.forms.utils import pretty_name

    utils.inject_class(forms, "BoundField", BoundField)
    utils.inject_callable(forms, "pretty_name", pretty_name)


@django1_31_bc_fixer()
def fix_deletion_forms_fields_EMPTY_VALUES(utils):
    """
    Preserve the compatibility import of django.core.validators.EMPTY_VALUES in django.forms.fields
    """
    from django.core.validators import EMPTY_VALUES
    from django.forms import fields
    utils.inject_attribute(fields, "EMPTY_VALUES", EMPTY_VALUES)


@django1_31_bc_fixer()
def fix_deletion_template_base_Context_classes(utils):
    """
    Preserve the compatibility imports django.template.Context, django.template.RequestContext
    and django.template.ContextPopException
    """
    from django.template.context import (Context, RequestContext, ContextPopException)
    from django.template import base
    utils.inject_class(base, "Context", Context)
    utils.inject_class(base, "RequestContext", RequestContext)
    utils.inject_class(base, "ContextPopException", ContextPopException)


@django1_31_bc_fixer()
def fix_deletion_core_management_commands_runserver(utils):
    """
    Preserve the compatibility alias django.core.management.commands.runserver.BaseRunserverCommand
    """
    from django.core.management.commands import runserver
    utils.inject_class(runserver, "BaseRunserverCommand", runserver.Command)


@django1_31_bc_fixer()
def fix_deletion_utils_decorators_classproperty(utils):
    """
    Preserve django.utils.decorators.classproperty as alias of new django.utils.functional.classproperty
    """
    from django.utils.functional import classproperty
    from django.utils import decorators
    utils.inject_class(decorators, "classproperty", classproperty)


@django1_31_bc_fixer()
def fix_deletion_contrib_admin_ACTION_CHECKBOX_NAME(utils):
    """
    Preserve import of ACTION_CHECKBOX_NAME in django.contrib.admin
    """
    from django.contrib.admin.helpers import ACTION_CHECKBOX_NAME
    from django.contrib import admin

    utils.inject_attribute(admin, "ACTION_CHECKBOX_NAME", ACTION_CHECKBOX_NAME)
    admin.__all__.append("ACTION_CHECKBOX_NAME")


@django1_31_bc_fixer()
def fix_deletion_views_debug_ExceptionReporterFilter(utils):
    """
    Preserve undocumented ExceptionReporterFilter class
    """
    from django.views import debug as debug_module

    class ExceptionReporterFilter:
        """
        Base for all exception reporter filter classes. All overridable hooks
        contain lenient default behaviors.
        """

        def get_post_parameters(self, request):
            if request is None:
                return {}
            else:
                return request.POST

        def get_traceback_frame_variables(self, request, tb_frame):
            return list(tb_frame.f_locals.items())

    # Beware, it's not the parent of SafeExceptionReporterFilter anymore, but it shouldn't be a problem
    utils.inject_class(debug_module, "ExceptionReporterFilter", ExceptionReporterFilter)

@django1_31_bc_fixer()
def fix_deletion_contrib_postgres_forms_jsonb_InvalidJSONInput_JSONString(utils):
    """[UNSAFE] Preserve undocumented InvalidJSONInput and JSONString classes in django.contrib.postgres.forms.jsonb

    Requires psycopg2 to be installed.
    """

    from django.contrib.postgres.forms import jsonb
    from django.forms.fields import InvalidJSONInput, JSONString

    utils.inject_class(jsonb, "InvalidJSONInput", InvalidJSONInput)
    utils.inject_class(jsonb, "JSONString", JSONString)
    # We do not alter __all__ here, these were private utilities!


@django1_31_bc_fixer()
def fix_deletion_contrib_postgres_fields_jsonb_JsonAdapter(utils):
    """[UNSAFE] Preserve undocumented JsonAdapter class in django.contrib.postgres.fields.jsonb

    Requires psycopg2 to be installed.
    """
    from django.contrib.postgres.fields import jsonb

    import json
    from psycopg2.extras import Json

    class JsonAdapter(Json):
        """
        Customized psycopg2.extras.Json to allow for a custom encoder.
        """

        def __init__(self, adapted, dumps=None, encoder=None):
            self.encoder = encoder
            super().__init__(adapted, dumps=dumps)

        def dumps(self, obj):
            options = {'cls': self.encoder} if self.encoder else {}
            return json.dumps(obj, **options)

    utils.inject_class(jsonb, "JsonAdapter", JsonAdapter)
    # We do not alter __all__ here, it was a private utility!


from __future__ import absolute_import, print_function, unicode_literals

import _test_utilities
import pytest

import django_compat_patcher
from django_compat_patcher import default_settings
from django_compat_patcher.registry import django_patching_registry


def test_fix_incoming_urls_submodule():
    from django.urls import (
        get_callable,
        RegexURLPattern,
        RegexURLResolver,
        NoReverseMatch,
    )

    assert get_callable


def test_fix_deletion_templatetags_future():
    from compat import render_to_string
    from django.templatetags.future import cycle, firstof

    rendered = render_to_string("core_tags/test_future_cycle_and_firstof.html")
    assert rendered.strip() == "row1\nA"


def test_fix_deletion_template_defaulttags_ssi():
    # already tested in "test_fix_deletion_templatetags_future_ssi()"
    from django.template.defaulttags import ssi

    assert callable(ssi)


def test_fix_behaviour_urls_resolvers_RegexURLPattern():

    from django.core.urlresolvers import RegexURLPattern

    has_lookup_str = hasattr(RegexURLPattern, "lookup_str")

    def dummy_view(request):
        return 72627

    pattern = RegexURLPattern("homepage/", dummy_view)
    assert pattern.callback is dummy_view
    pattern.add_prefix("muyprefix")
    assert pattern.callback is dummy_view
    if has_lookup_str:
        assert pattern.lookup_str.endswith(".dummy_view")  # complex on py3k

    pattern = RegexURLPattern("homepage/", "test_project.views.my_view")
    assert pattern.callback.__name__ == "my_view"
    if has_lookup_str:
        assert pattern.lookup_str == "test_project.views.my_view"
    pattern.add_prefix("myprefix")
    try:
        pattern.callback  # bad prefix now
    except ImportError:
        pass  # might or not raise, depending on django version (caching or not)
    if has_lookup_str:
        # our own "lookup_str" property bypasses the original, CACHED, one
        assert pattern.lookup_str == "myprefix.test_project.views.my_view"

    pattern = RegexURLPattern("homepage/", "my_view")
    with pytest.raises(ImportError):
        myvar = pattern.callback  # missing prefix
    if has_lookup_str:
        assert pattern.lookup_str == "my_view"  # missing prefix but works
    pattern.add_prefix("test_project.views")
    assert pattern.callback.__name__ == "my_view"
    if has_lookup_str:
        assert pattern.lookup_str == "test_project.views.my_view"


def test_fix_behaviour_core_urlresolvers_reverse_with_prefix():
    try:
        from django.core.urlresolvers import reverse
    except ImportError:
        from django.urls import reverse

    view = reverse("homepage")  # by view name
    assert view == "/homepage/"

    view = reverse("test_project.views.my_view")  # by dotted path
    assert view == "/my_view/"


def test_fix_behaviour_conf_urls_url():
    from django.conf.urls import url

    url(r"^admin2/", "test_project.views.my_view", name="test_admin_abc"),


def test_fix_deletion_conf_urls_patterns():
    import django.conf.urls
    from django.conf.urls import patterns, url

    patterns(
        "admin",
        (r"^admin1/", "test_project.views.my_view"),
        url(r"^admin2/", "test_project.views.my_view", name="test_admin_other"),
    )
    assert "patterns" in django.conf.urls.__all__


def test_fix_behaviour_template_smartif_OPERATORS_equals():
    from compat import render_to_string

    rendered = render_to_string("core_tags/test_smartif_operators.html", dict(a=3))
    assert rendered.strip() == "hello\nbye"


def test_fix_behaviour_core_management_parser_optparse():
    from django.core import management
    from six import StringIO
    from django.test.utils import captured_stderr, captured_stdout

    with captured_stdout() as stdout, captured_stderr():
        # Check that new, argparse-based commands, still work as intended!
        management.execute_from_command_line(['django-admin', 'diffsettings'])
    stdout_value = stdout.getvalue()
    print(stdout_value)
    assert "ALLOWED_HOSTS" in stdout_value

    # Check that optparse-based command works
    out = StringIO()
    management.call_command('optparse_cmd', stdout=out)
    assert out.getvalue() == "All right, let's dance Rock'n'Roll.\n"

    # Simulate command line execution
    with captured_stdout() as stdout, captured_stderr():
        # We skip checks since test project is not complete
        needs_skip_checks = (_test_utilities.DJANGO_VERSION_TUPLE >= (1, 10))
        cmd_line = ['django-admin', 'optparse_cmd'] + (["--skip-checks"] if needs_skip_checks else [])
        management.execute_from_command_line(cmd_line)
    stdout_value = stdout.getvalue()
    print(stdout_value)
    assert stdout_value == "All right, let's dance Rock'n'Roll.\n"

    # This fixer is UNSAFE!
    fixer_id = "fix_behaviour_core_management_parser_optparse"
    assert django_patching_registry.get_fixer_by_id(fixer_id)
    assert fixer_id in default_settings.DCP_EXCLUDE_FIXER_IDS



import os

import pytest
from django.conf import settings
from django_compat_patcher.config import DjangoSettingsProvider

from compat_patcher_core.scaffolding import ensure_no_stdlib_warnings


def test_ensure_no_stdlib_warnings_in_package():
    import warnings  # This line will trigger checker error
    del warnings

    import django_compat_patcher

    pkg_root = os.path.dirname(django_compat_patcher.__file__)
    analysed_files = ensure_no_stdlib_warnings(pkg_root)
    assert len(analysed_files) >= 3, analysed_files

    test_root = os.path.dirname(__file__)
    with pytest.raises(ValueError, match="wrong phrase.*test_scaffolding.py"):
        ensure_no_stdlib_warnings(test_root)


def test_no_package_shadowing_in_tox():
    import django_compat_patcher

    package_dir = os.path.dirname(os.path.abspath(django_compat_patcher.__file__))
    if os.getenv("INSIDE_TOX_FOR_DCP") and ".tox" not in package_dir:
        raise RuntimeError("Wrong django_compat_patcher package used in Tox")


def test_dcp_settings_overrides():

    old_environ = os.environ.copy()

    assert not hasattr(settings, "DCP_INCLUDE_FIXER_IDS")
    settings.DCP_INCLUDE_FIXER_IDS = ["some_django_setting_for_included_ids"]

    try:

        django_settings_provider = DjangoSettingsProvider(settings=dict(DCP_INCLUDE_FIXER_FAMILIES="*"))
        assert django_settings_provider["include_fixer_families"] == "*"
        assert django_settings_provider["include_fixer_ids"] == "*"  # DEFAULT DCP VALUE, no fallback to Django-level setting

        django_settings_provider = DjangoSettingsProvider(settings=None)
        assert django_settings_provider["include_fixer_ids"] == ["some_django_setting_for_included_ids"]

        os.environ.update(
            DCP_INCLUDE_FIXER_IDS = '["some_included_id"]',
            DCP_INCLUDE_FIXER_FAMILIES = '["some_included_family"]',

            DCP_EXCLUDE_FIXER_IDS = '["some_excluded_id"]',
            DCP_EXCLUDE_FIXER_FAMILIES = '["some_excluded_family"]',

            DCP_PATCH_INJECTED_OBJECTS = 'true',

            DCP_LOGGING_LEVEL = '"DEBUG"',
            DCP_ENABLE_WARNINGS = 'false',
        )

        assert django_settings_provider["include_fixer_ids"] == ["some_included_id"]
        assert django_settings_provider["include_fixer_families"] == ["some_included_family"]
        assert django_settings_provider["exclude_fixer_ids"] == ["some_excluded_id"]
        assert django_settings_provider["exclude_fixer_families"] == ["some_excluded_family"]
        assert django_settings_provider["patch_injected_objects"] == True
        assert django_settings_provider["logging_level"] == "DEBUG"
        assert django_settings_provider["enable_warnings"] == False

        del os.environ["DCP_INCLUDE_FIXER_IDS"]
        assert django_settings_provider["include_fixer_ids"] == ["some_django_setting_for_included_ids"]  # Fallback to django-level setting

        os.environ.update(
            DCP_INCLUDE_FIXER_IDS = "['some_included_id']"  # NOT real json due to quotes
        )
        with pytest.raises(ValueError, match="Abnormal json"):
            django_settings_provider["include_fixer_ids"]
    finally:
        os.environ.clear()
        os.environ.update(old_environ)
        del settings.DCP_INCLUDE_FIXER_IDS





import sys

import six
import pytest


skip_on_python2 = pytest.mark.skipif(sys.version_info < (3,), reason="This Django3 fixer requires python3 or higher")


def test_fix_deletion_utils_six():
    import django.utils.six
    from django.utils import six as six2
    assert django.utils.six.string_types
    assert six2.string_types

    from django.utils.encoding import six as six3
    assert isinstance(six3.PY2, bool)
    assert six3 is six2


def test_fix_deletion_utils_upath_npath_abspathu():
    from os.path import abspath
    from django.utils._os import abspathu, upath, npath

    assert abspathu(".") == abspath(".")
    assert upath("/something/file.txt") == "/something/file.txt"  # No-op
    assert npath("/something2/file.txt") == "/something2/file.txt"  # No-op


@skip_on_python2
def test_fix_deletion_utils_decorators_ContextDecorator():
    from django.utils.decorators import ContextDecorator
    from contextlib import ContextDecorator as ContextDecoratorOriginal
    assert isinstance(ContextDecorator, type)
    assert ContextDecorator is ContextDecoratorOriginal


def test_fix_deletion_utils_decorators_available_attrs():
    from django.utils.decorators import available_attrs
    from functools import WRAPPER_ASSIGNMENTS

    def func():
        pass

    assert available_attrs(func) == WRAPPER_ASSIGNMENTS


@skip_on_python2
def test_fix_deletion_utils_lru_cache_lru_cache():
    from django.utils.lru_cache import lru_cache as django_lru_cache_function
    from functools import lru_cache
    assert callable(django_lru_cache_function)
    assert django_lru_cache_function is lru_cache


@skip_on_python2
def test_fix_deletion_utils_safestring_SafeBytes():
    from django.utils.safestring import SafeBytes
    assert SafeBytes("abc", "ascii") == bytes("abc", "ascii")


@skip_on_python2
def test_fix_deletion_test_utils_str_prefix():
    from django.test.utils import str_prefix
    assert str_prefix("%(_)shello") == "hello"


def test_fix_deletion_test_utils_patch_logger():
    import logging
    from django.test.utils import patch_logger
    with patch_logger("django", "info") as calls:
        logging.getLogger("django").info("Patch-logger context manager seems to work %s", "fine")
    assert len(calls) == 1
    assert calls == ["Patch-logger context manager seems to work fine"]


@skip_on_python2
def test_fix_deletion_utils_encoding_python_2_unicode_compatible():
    from django.utils.encoding import python_2_unicode_compatible

    @python_2_unicode_compatible
    class MyClass:
        def __str__(self):
            return "<Some MyClass object>"

    obj = MyClass()
    assert isinstance(obj, MyClass)


def test_fix_deletion_utils_functional_curry():

    from django.utils.functional import curry

    def func(value1, value2, value3, value4):
        return value1 + value2 + value3 + value4

    func2 = curry(func, "hello", value3="general")
    assert func2("there", value4="kenobi") == "hellotheregeneralkenobi"


def test_fix_deletion_shortcuts_render_to_response():
    from django.shortcuts import render_to_response
    res = render_to_response("example_template.html", context=dict(message="ThiIsATest"),
                             content_type="dummycontenttype", status=201)
    assert res['Content-Type'] == "dummycontenttype"
    assert res.status_code == 201
    assert six.b("ThiIsATest") in res.content




from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_11_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="1.11",
    fixer_applied_from_version="1.11",
)


@register_django_compatibility_fixer(
    fixer_reference_version="1.11", fixer_applied_upto_version="1.11"
)
def fix_incoming_test_utils_setup_test_environment_signature_change(utils):
    """
    Set a forward compatibility wrapper for setup_test_environment() which takes a "debug" argument later.
    """
    import inspect
    from django.test import utils as django_utils

    original_setup_test_environment = django_utils.setup_test_environment

    # Sanity check
    argspec = inspect.getargspec(original_setup_test_environment)
    assert not argspec.args, argspec

    def setup_test_environment(debug=None):
        return original_setup_test_environment()  # No params

    utils.inject_callable(django_utils, "setup_test_environment", setup_test_environment)


@django1_11_bc_fixer()
def fix_behaviour_widget_build_attrs(utils):
    """
    Preserve compatibility with the old signature of Widget.build_attrs(): extra_attrs=None, **kwargs.
    """
    from django.forms.widgets import Widget

    # original_build_attrs = Widget.build_attrs  UNUSED

    def build_attrs(self, base_attrs=None, extra_attrs=None, **kwargs):
        if kwargs:
            # old version
            assert not extra_attrs
            attrs = dict(self.attrs, **kwargs)
            if base_attrs:  # actually the old "extra_attrs"
                attrs.update(base_attrs)
        else:
            # new version
            attrs = base_attrs.copy()
            if extra_attrs is not None:
                attrs.update(extra_attrs)
        return attrs

    utils.inject_callable(Widget, "build_attrs", build_attrs)



import pytest

import _test_utilities

from django_compat_patcher import default_settings
from django_compat_patcher.registry import django_patching_registry


def test_fix_deletion_db_models_submodules_EmptyResultSet():
    from django.db.models.query import EmptyResultSet as EmptyResultSet1
    from django.db.models.sql import EmptyResultSet as EmptyResultSet2
    from django.db.models.sql.datastructures import EmptyResultSet as EmptyResultSet3

    assert EmptyResultSet1 is EmptyResultSet2
    assert EmptyResultSet2 is EmptyResultSet3

    import django.db.models.sql
    assert "EmptyResultSet" in django.db.models.sql.__all__


def test_fix_deletion_db_models_fields_FieldDoesNotExist():
    from django.db.models.fields import FieldDoesNotExist
    assert FieldDoesNotExist

    from django.db.models import fields
    assert "FieldDoesNotExist" in fields.__all__


def test_fix_deletion_forms_forms_pretty_name_BoundField():
    from django.forms.forms import BoundField
    assert BoundField
    from django.forms.forms import pretty_name
    assert pretty_name("my_dog") == "My dog"


def test_fix_deletion_forms_fields_EMPTY_VALUES():
    from django.forms.fields import EMPTY_VALUES
    assert EMPTY_VALUES


def test_fix_deletion_template_base_Context_classes():
    from django.template.base import (Context, RequestContext, ContextPopException)
    assert Context
    assert RequestContext
    assert ContextPopException


def test_fix_deletion_core_management_commands_runserver():
    from django.core.management.commands.runserver import Command, BaseRunserverCommand
    assert Command is BaseRunserverCommand


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 9),
    reason="requires django.utils.decorators.classproperty introduction",
)
def test_fix_deletion_utils_decorators_classproperty():
    from django.utils.decorators import classproperty

    class MyClass:
        @classproperty
        def myfunc(cls):
            return 43

    assert MyClass.myfunc == 43
    assert MyClass().myfunc == 43


def test_fix_deletion_contrib_admin_ACTION_CHECKBOX_NAME():
    from django.contrib.admin import ACTION_CHECKBOX_NAME, autodiscover
    del autodiscover  # Proper module is targeted
    assert ACTION_CHECKBOX_NAME == "_selected_action"

    import django.contrib.admin
    assert "ACTION_CHECKBOX_NAME" in django.contrib.admin.__all__


def test_fix_deletion_views_debug_ExceptionReporterFilter():
    from django.views.debug import ExceptionReporterFilter
    filter = ExceptionReporterFilter()
    assert filter.get_post_parameters(None) == {}


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 11),
    reason="requires posgres forms jsonb module",
)
def test_fix_deletion_contrib_postgres_forms_jsonb_InvalidJSONInput_JSONString():

    # We expect psycopg2 to be installed here

    # These upper level imports NEVER existed!
    with pytest.raises(ImportError):
        from django.contrib.postgres.forms import InvalidJSONInput
    with pytest.raises(ImportError):
        from django.contrib.postgres.forms import JSONString

    from django.contrib.postgres.forms.jsonb import InvalidJSONInput, JSONString

    assert InvalidJSONInput("hello")
    assert JSONString("something")

    try:
        from django.forms.fields import InvalidJSONInput as _InvalidJSONInput, JSONString as _JSONString
    except ImportError:
        pass  # It's OK, these do not exist yet
    else:
        assert _InvalidJSONInput is InvalidJSONInput
        assert _JSONString is JSONString

    # This fixer is UNSAFE since it requires psycopg2
    fixer_id = "fix_deletion_contrib_postgres_forms_jsonb_InvalidJSONInput_JSONString"
    assert django_patching_registry.get_fixer_by_id(fixer_id)
    assert fixer_id in default_settings.DCP_EXCLUDE_FIXER_IDS


@pytest.mark.skipif(
    _test_utilities.DJANGO_VERSION_TUPLE < (1, 11),
    reason="requires posgres fields jsonb module",
)
def test_fix_deletion_contrib_postgres_fields_jsonb_JsonAdapter():

    from django.contrib.postgres.fields.jsonb import JsonAdapter
    assert JsonAdapter.dumps  # Hard to test here...

    # This fixer is UNSAFE since it requires psycopg2
    fixer_id = "fix_deletion_contrib_postgres_fields_jsonb_JsonAdapter"
    assert django_patching_registry.get_fixer_by_id(fixer_id)
    assert fixer_id in default_settings.DCP_EXCLUDE_FIXER_IDS


#!/usr/bin/env python

import sys, os

os.chdir(os.path.dirname(os.path.abspath(__file__)))  # security

from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read().strip()


classifiers = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
Intended Audience :: Information Technology
Intended Audience :: System Administrators
License :: OSI Approved :: MIT License
Programming Language :: Python
Programming Language :: Python :: 2.7
Programming Language :: Python :: 3.4
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.7
Framework :: Django
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Microsoft :: Windows
Operating System :: Unix
Operating System :: MacOS :: MacOS X
"""

packages = find_packages(where="src")

needs_pytest = {"pytest", "test", "ptr"}.intersection(sys.argv)
setup_requires = ["pytest-runner"] if needs_pytest else []

extras = {"comments": ["django-contrib-comments"]}

setup(
    name="django-compat-patcher",
    version=read("VERSION"),
    author="Pascal Chambon & others",
    author_email="pythoniks@gmail.com",
    url="https://github.com/pakal/django-compat-patcher",
    license="MIT",
    platforms=["any"],
    description="A monkey-patcher to ease the transition of project to latest Django versions.",
    classifiers=filter(None, classifiers.split("\n")),
    long_description=read("README.rst"),
    package_dir={"": "src"},
    packages=packages,
    install_requires=['compat-patcher-core',
                      'six',
                      'Django<2;python_version<"3.0"',
                      'Django;python_version>="3.0"'],
    extras_require=extras,
    setup_requires=setup_requires,
    tests_require=[
        "pytest",
        "pytest-cov",
        "django-compat",
    ],  # Beware, keep in sync with tox.ini
    use_2to3=True,
    # convert_2to3_doctests=['src/your/module/README.txt'],
    # use_2to3_fixers=['your.fixers'],
    use_2to3_exclude_fixers=["lib2to3.fixes.fix_import"],
)



from __future__ import absolute_import, print_function, unicode_literals

from functools import partial

from ..deprecation import *
from ..registry import register_django_compatibility_fixer

# for backward-compatibility fixers
django1_30_bc_fixer = partial(
    register_django_compatibility_fixer,
    fixer_reference_version="3.0",
    fixer_applied_from_version="3.0",
)


@django1_30_bc_fixer()
def fix_deletion_utils_six(utils):
    """Preserve the vendored copy of "six" compatibility utility, in django.utils,
    as well as the `six` import in django.utils.encoding"""
    import six
    utils.inject_import_alias(alias_name="django.utils.six", real_name="six")

    import sys, six
    from django.utils import encoding
    if not hasattr(encoding, "six"):
        utils.inject_attribute(encoding, "six", six)


@django1_30_bc_fixer()
def fix_deletion_utils_upath_npath_abspathu(utils):
    """Preserve python2 path normalization functions."""

    from os.path import abspath as abspathu  # For backwards-compatibility in Django 2.0

    def upath(path):
        """Always return a unicode path (did something for Python 2)."""
        return path

    def npath(path):
        """
        Always return a native path, that is unicode on Python 3 and bytestring on
        Python 2. Noop for Python 3.
        """
        return path

    from django.utils import _os
    utils.inject_callable(_os, "abspathu", abspathu)
    utils.inject_callable(_os, "upath", upath)
    utils.inject_callable(_os, "npath", npath)


@django1_30_bc_fixer()
def fix_deletion_utils_decorators_ContextDecorator(utils):
    """Preserve django.utils.decorators.ContextDecorator, alias of contextlib.ContextDecorator."""
    from contextlib import ContextDecorator
    from django.utils import decorators
    utils.inject_class(decorators, "ContextDecorator", ContextDecorator)


@django1_30_bc_fixer()
def fix_deletion_utils_decorators_available_attrs(utils):
    """Preserve django.utils.decorators.available_attrs, which just returns functools.WRAPPER_ASSIGNMENTS."""

    def available_attrs(fn):  # For backwards compatibility in Django 2.0.
        """
        Return the list of functools-wrappable attributes on a callable.
        This was required as a workaround for https://bugs.python.org/issue3445
        under Python 2.
        """
        from functools import WRAPPER_ASSIGNMENTS
        return WRAPPER_ASSIGNMENTS

    from django.utils import decorators
    utils.inject_callable(decorators, "available_attrs", available_attrs)


@django1_30_bc_fixer()
def fix_deletion_utils_lru_cache_lru_cache(utils):
    """Preserve django.utils.lru_cache.lru_cache(), alias of functools.lru_cache(), and its containing module."""
    from ..django_legacy.django3_0 import lru_cache as django_lru_cache_module
    utils.inject_module("django.utils.lru_cache", django_lru_cache_module)


@django1_30_bc_fixer()
def fix_deletion_utils_safestring_SafeBytes(utils):
    """Preserve django.utils.safestring.SafeBytes class."""
    from django.utils.safestring import SafeData, SafeText
    from django.utils import safestring

    class SafeBytes(bytes, SafeData):
        """
        A bytes subclass that has been specifically marked as "safe" (requires no
        further escaping) for HTML output purposes.

        Kept in Django 2.0 for usage by apps supporting Python 2. Shouldn't be used
        in Django anymore.
        """
        def __add__(self, rhs):
            """
            Concatenating a safe byte string with another safe byte string or safe
            string is safe. Otherwise, the result is no longer safe.
            """
            t = super(SafeBytes, self).__add__(rhs)
            if isinstance(rhs, SafeText):
                return SafeText(t)
            elif isinstance(rhs, SafeBytes):
                return SafeBytes(t)
            return t

    utils.inject_class(safestring, "SafeBytes", SafeBytes)


@django1_30_bc_fixer()
def fix_deletion_test_utils_str_prefix(utils):
    """Preserve django.test.utils.str_prefix class."""
    from django.test import utils as test_utils
    def str_prefix(s):
        return s % {'_': ''}
    utils.inject_callable(test_utils, "str_prefix", str_prefix)


@django1_30_bc_fixer()
def fix_deletion_test_utils_patch_logger(utils):
    """Preserve django.test.utils.patch_logger() context manager."""
    from contextlib import contextmanager
    import logging
    from django.test import utils as test_utils

    @contextmanager
    def patch_logger(logger_name, log_level, log_kwargs=False):
        """
        Context manager that takes a named logger and the logging level
        and provides a simple mock-like list of messages received.

        Use unittest.assertLogs() if you only need Python 3 support. This
        private API will be removed after Python 2 EOL in 2020 (#27753).
        """
        calls = []

        def replacement(msg, *args, **kwargs):
            call = msg % args
            calls.append((call, kwargs) if log_kwargs else call)
        logger = logging.getLogger(logger_name)
        orig = getattr(logger, log_level)
        setattr(logger, log_level, replacement)
        try:
            yield calls
        finally:
            setattr(logger, log_level, orig)
    utils.inject_callable(test_utils, "patch_logger", patch_logger)


@django1_30_bc_fixer()
def fix_deletion_utils_encoding_python_2_unicode_compatible(utils):
    """Preserve django.utils.encoding.python_2_unicode_compatible() class decorator."""
    import six
    from django.utils import encoding
    utils.inject_callable(encoding, "python_2_unicode_compatible", six.python_2_unicode_compatible)


@django1_30_bc_fixer()
def fix_deletion_utils_functional_curry(utils):
    """Preserve django.utils.functional.curry()function."""

    from django.utils import functional
    # You can't trivially replace this with `functools.partial` because this binds
    # to classes and returns bound instances, whereas functools.partial (on
    # CPython) is a type and its instances don't bind. --> see `functools.partialmethod` now!
    def curry(_curried_func, *args, **kwargs):
        def _curried(*moreargs, **morekwargs):
            allkwargs = dict(**kwargs)
            allkwargs.update(morekwargs)
            return _curried_func(*(args + moreargs), **allkwargs)
        return _curried

    utils.inject_callable(functional, "curry", curry)


@django1_30_bc_fixer()
def fix_deletion_shortcuts_render_to_response(utils):
    """Preserve django.shortcuts.render_to_response(), superseded by render()."""
    from django import shortcuts

    def render_to_response(template_name, context=None, content_type=None, status=None, using=None):
        """
        Return a HttpResponse whose content is filled with the result of calling
        django.template.loader.render_to_string() with the passed arguments.
        """
        warnings.warn(
            'render_to_response() is deprecated in favor of render(). It has the '
            'same signature except that it also requires a request.',
            RemovedInDjango30Warning, stacklevel=2,
        )
        from django.template import loader
        from django.http import HttpResponse
        content = loader.render_to_string(template_name, context, using=using)
        return HttpResponse(content, content_type, status)

    utils.inject_callable(shortcuts, "render_to_response", render_to_response)
